<?php
/**
 * Copyright 2012 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage Rating
 */

/**
 * Form to submit a customer rating
 *
 * @package Silvercart
 * @subpackage Rating
 * @author Patrick Schneider <pschneider@pixeltricks.de>
 * @copyright 2012 pixeltricks GmbH
 * @since 06.05.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartCustomerRatingForm extends CustomHtmlForm {
    
    /**
     * Indicator to check whether to build the formfields or not
     *
     * @var bool
     */
    protected $getFormFieldsIsCalled = false;

    /**
     * definition of the form fields
     *
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 27.07.2012
     */
    public function getFormFields() {
        if (!$this->getFormFieldsIsCalled) {
            $rating = singleton('SilvercartCustomerRating');

            $this->formFields = array(
                'Rating' => array(
                    'type' => 'DropdownField',
                    'title' => $rating->fieldLabel('Rating'),
                    'value' => array(
                        ''  => '',
                        '1' => '1',
                        '2' => '2',
                        '3' => '3',
                        '4' => '4',
                        '5' => '5',
                    ),
                    'checkRequirements' => array(
                        'isFilledIn' => true
                    )
                ),
                'Author' => array(
                    'type' => 'TextField',
                    'title' => $rating->fieldLabel('Author'),
                    'checkRequirements' => array(
                        'isFilledIn' => true,
                        'hasMinLength' => 3
                    )
                ),
                'Message' => array(
                    'type' => 'TextareaField',
                    'title' => $rating->fieldLabel('Message'),
                    'checkRequirements' => array(
                        'isFilledIn' => true,
                        'hasMinLength' => 3
                    )
                )
            );
            $this->getFormFieldsIsCalled = true;
        }
        return parent::getFormFields();
    }
    
    /**
     * Preferences for this form
     *
     * @return void
     *
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 07.05.2012 
     */
    public function preferences() {
        $this->preferences = array(
            'submitButtonTitle'  => _t('SilvercartCustomerRatingForm.SUBMITBUTTONTITLE'),
        );
        parent::preferences();
    }
    
    /**
     * used to fill field values and to translate the labels
     * 
     * @return void
     * 
     * @author Patrick Schneider <pschneider@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 12.12.2012
     */
    protected function fillInFieldValues() {
        parent::fillInFieldValues();
        $customer = Member::currentUser();
        if ($customer &&
            (!array_key_exists('value', $this->formFields['Author']) ||
             empty($this->formFields['Author']['value']))) {
            $this->formFields['Author']['value'] = $customer->getName();
        }
        
    }
    
    /**
     * gets called after successful validation
     * 
     * @param SS_HTTPRequest $data     contains the frameworks form data
     * @param Form           $form     Form
     * @param array          $formData contains the modules form data
     * 
     * @return void
     * 
     * @author Patrick Schneider <pschneider@pixeltricks.de>
     * @since 03.04.2012 
     */
    protected function submitSuccess($data, $form, $formData) {
        $formData['SilvercartProductID']    = $this->customParameters['SilvercartProductID'];
        $formData['Locale']                 = Translatable::get_current_locale();
        $formData['Message']                = str_replace('\r\n', PHP_EOL, $formData['Message']);
        $rating = new SilvercartCustomerRating();
        $rating->update($formData);
        $rating->write();
        if (!Director::redirected_to()) {
            Director::redirectBack();
        }
    }
}