<?php
/**
 * Copyright 2012 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage Rating
 */

/**
 * Plugin to add customer ratings into a product tab
 * 
 * @package Silvercart
 * @subpackage Rating
 * @author Patrick Schneider <pschneider@pixeltricks.de>
 * @copyright 2012 pixeltricks GmbH
 * @since 03.04.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartCustomerRatingProductPlugin extends DataObjectDecorator {
    
    /**
     * returns a rating related to a product as dataobject
     * 
     * @param Object &$callingObject calling object
     * 
     * @return DataObject
     * 
     * @author Patrick Schneider <pschneider@pixeltricks.de>
     * @since 03.04.2012
     */
    public function pluginGetPluggedInTabs(&$callingObject) {
        $data = array(
            'Name'      => _t('SilvercartCustomerRating.CUSTOMERREVIEWS'),
            'Content'   => $callingObject->renderWith('SilvercartCustomerRatings'),
            'TabID'     => 'CustomerRatingsTab',
        );
        $dataObject = new DataObject($data);
        return $dataObject;
    }
    
    /**
     * Returns the average customer rating as rounded value in a dataobject
     * 
     * @param Object &$callingObject calling object
     * 
     * @return DataObject
     * 
     * @author Patrick Schneider <pschneider@pixeltricks.de>
     * @since 03.04.2012 
     */
    public function pluginGetPluggedInProductMetaData(&$callingObject) {
        $data = array(
            'MetaData' => $callingObject->renderWith('SilvercartCustomerRatingsMetaData'),
        );
        $dataObject = new DataObject($data);
        return $dataObject;
    }
}