<?php
/**
 * Copyright 2012 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package Silvercart
 * @subpackage Rating
 */

/**
 * A task to clean the products average rating and rating count
 *
 * @package Silvercart
 * @subpackage Rating
 * @author Sebastian Diel <sdiel@pixeltricks.de>
 * @copyright 2012 pixeltricks GmbH
 * @since 23.03.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
class SilvercartCustomerRatingTask extends ScheduledTask {
    
    /**
     * calculates the average rating of product when a new rating is added
     * 
     * @return void
     * 
     * @author Sebastian Diel <sdiel@pixeltricks.de>
     * @since 24.07.2012 
     */
    public function process() {
        $products = DataObject::get('SilvercartProduct');
        if ($products) {
            foreach ($products as $product) {
                $ratings                = $product->SilvercartCustomerRatings();
                $avarageCustomerRating  = 0;
                if ($ratings->Count() > 0) {
                    foreach ($ratings as $rating) {
                        $avarageCustomerRating += $rating->Rating;
                    }
                    $avarageCustomerRating = round($avarageCustomerRating / $ratings->Count(), 2);
                }
                $beforeAvarageCustomerRating    = $product->AvarageCustomerRating;
                $beforeCustomerRatingCount      = $product->CustomerRatingCount;
                $product->AvarageCustomerRating = $avarageCustomerRating;
                $product->CustomerRatingCount   = $ratings->Count();
                $product->write();
                if ($beforeAvarageCustomerRating != $product->AvarageCustomerRating ||
                    $beforeCustomerRatingCount != $product->CustomerRatingCount) {
                    if (Director::is_cli()) {
                        print 'Updated product #' . $product->ID . PHP_EOL;
                        print "\t" . 'New AvarageCustomerRating: ' . $avarageCustomerRating . ' (before: ' . $beforeAvarageCustomerRating . ')' . PHP_EOL;
                        print "\t" . 'New CustomerRatingCount: ' . $ratings->Count() . ' (before: ' . $beforeCustomerRatingCount . ')' . PHP_EOL;
                        print PHP_EOL;
                    }
                }
            }
        }
    }
    
}