<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 */

/**
 * Is used as storage for user input and gets appended to a shoppingcart
 * position.
 *
 * @package SilvercartProductVariants
 * @subpackage Base
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 25.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantUserGeneratedAttribute extends DataObject {
    
    /**
     * Attributes.
     *
     * @var array
     */
    public static $db = array(
        'inputObjectClassName'  => 'VarChar(255)',
        'inputObjectID'         => 'Int'
    );
    
    /**
     * Belongs-many-many relationships.
     *
     * @var array
     */
    public static $belongs_many_many = array(
        'SilvercartShoppingCartPositions' => 'SilvercartShoppingCartPosition'
    );
    
    /**
     * We want to delete the attributed object here.
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 30.11.2011
     */
    public function onBeforeDelete() {
        $sourceClassObj  = DataObject::get_by_id($this->inputObjectClassName, $this->inputObjectID);
        
        if ($sourceClassObj) {
            $sourceClassObj->delete();
        }
        
        parent::onBeforeDelete();
    }
    
    /**
     * Returns the translated singular name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string The objects singular name 
     * 
     * @author Roland Lehmann <rlehmann@pixeltricks.de>
     * @since 14.07.2012
     */
    public function singular_name() {
        return SilvercartTools::singular_name_for($this);
    }


    /**
     * Returns the translated plural name of the object. If no translation exists
     * the class name will be returned.
     * 
     * @return string the objects plural name
     * 
     * @author Roland Lehmann <rlehmann@pixeltricks.de>
     * @since 14.07.2012
     */
    public function plural_name() {
        return SilvercartTools::plural_name_for($this); 
    }
    
    /**
     * Summaryfields for display in tables.
     *
     * @return array
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 25.11.2011
     */
    public function summaryFields() {
        $summaryFields = array(
            'name' => _t('SilvercartProductVariantUserGeneratedAttribute.NAME_LABEL')
        );
        
        $this->extend('updateSummaryFields', $summaryFields);
        return $summaryFields;
    }
}