<?php
/**
 * Copyright 2012 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariant
 * @subpackage FormFields
 */

/**
 * Provides buttons with actions for a attribute set.
 * 
 * @package SilvercartProductVariant
 * @subpackage FormFields
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @copyright 2012 pixeltricks GmbH
 * @since 13.01.2012
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 */
 class SilvercartVariantActionsField extends FormField {

    /**
     * The controller object.
     *
     * @var Object
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 13.01.2012
     */
    protected $controller;

    /**
     * The status message.
     *
     * @var string
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 13.01.2012
     */
    protected $status = '';
    
    /**
     * Constructor
     *
     * @param Controller $controller The controller object.
     * @param string     $name       The internal field name, passed to forms.
	 * @param string     $title      The field label.
	 * @param string     $status     The status message.
     * 
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 13.01.2012
     */
    public function __construct($controller, $name, $title = null, $status = '') {
        parent::__construct($name, $title);
        
        $this->status     = $status;
        $this->controller = $controller;
    }

    /**
     * Returns the rendered HTML code for this field.
     *
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 13.01.2012
     */
    public function FieldHolder() {
        $data    = array();

        $data['Actions'] = new DataObjectSet();
        $data['Actions']->push(
            new DataObject(
                array(
                    'Title'  => _t('SilvercartProductVariantAttributeSet.ACTIONS_FOR_SET'),
                    'name'   => _t('SilvercartProductVariantAttributeSet.UPDATE_ATTRIBUTED_SETS'),
                    'action' => 'action_doUpdateAttributedSets',
                    'rel'    => $this->controller->ID,
                    'status' => $this->status
                )
            )
        );

        return $this->customise($data)->renderWith($this->class);
    }
}

