<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 */

/**
 * Plugin for the SilvercartIncrementPositionQuantityForm object.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 23.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartIncrementPositionQuantityFormPlugin extends DataObjectDecorator {
    
    /**
     * Overwrites the original submitSucess method.
     *
     * @param array &$arguments     The arguments to pass
     *                              $arguments[0] = $data     contains the frameworks form data
     *                              $arguments[1] = $form     not used
     *                              $arguments[2] = $formData contains the modules form data
     * @param mixed &$callingObject The calling object
     * 
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 23.11.2011
     */
    public function pluginOverwriteSubmitSuccess(&$arguments, &$callingObject) {
        $data       = $arguments[0];
        $form       = $arguments[1];
        $formData   = $arguments[2];
        
        if ($formData['positionID']) {
            
            //check if the position belongs to this user. Malicious people could manipulate it.
            $member     = Member::currentUser();
            $position   = DataObject::get_by_id('SilvercartShoppingCartPosition', $formData['positionID']);
            
            if ($position && ($member->SilvercartShoppingCart()->ID == $position->SilvercartShoppingCartID)) {
                
                $attributes     = array();
                $ugAttributes   = array();
                
                foreach ($position->ProductVariantAttributes() as $attribute) {
                    $attributes[$attribute->SilvercartProductVariantAttributeSet] = $attribute->ID;
                }
                
                foreach ($position->ProductVariantUGAttributes() as $ugAttribute) {
                    $sourceClassName = $ugAttribute->inputObjectClassName;
                    $sourceClassID   = $ugAttribute->inputObjectID;
                    $sourceClassObj  = DataObject::get_by_id($sourceClassName, $sourceClassID);

                    if ($sourceClassObj) {
                        $existingGeneratedAttribute = DataObject::get_one(
                            'SilvercartProductVariantUserGeneratedAttribute',
                            sprintf(
                                "inputObjectClassName = '%s' AND inputObjectID = '%s'",
                                $sourceClassObj->class,
                                $sourceClassObj->ID
                            )
                        );
                        
                        if ($existingGeneratedAttribute) {
                            $ugAttributes[] = $existingGeneratedAttribute;
                        }
                    }
                }
                
                $position->SilvercartProduct()->SVTLFaddToCartWithAttributes($member->SilvercartShoppingCart()->ID, 1, $attributes, $ugAttributes);

                if (!Director::redirected_to($callingObject->controller->Link())) {
                    Director::redirect($callingObject->controller->Link());
                }
            }
        }
        
        return true;
    }
}
