<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 */

/**
 * Delivers additional information for the addCartForm CustomHtmlForm object of
 * the SilvercartProductGroupPage detail view.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 16.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantAddCartFormDetailPlugin extends DataObjectDecorator {
    
    /**
     * We inject our additional fields here.
     * 
     * @param array &$formFields    Form fields to manipulate
     * @param mixed &$callingObject The calling object
     * 
     * @return bool
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 26.06.2012
     */
    public function pluginUpdateFormFields(&$formFields, &$callingObject) {
        $silvercartProduct = Controller::curr()->detailViewProduct;
        
        $callingObject->fieldGroups['SilvercartProductVariantAttributeSets'] = array();
        $attributeSets = $silvercartProduct->SilvercartAttributedVariantAttributeSets();
        $attributeSets->sort('SortOrder', 'ASC');

        foreach ($attributeSets as $attributeSet) {
            
            // ----------------------------------------------------------------
            // Standard attributes with defined values
            // ----------------------------------------------------------------
            if ($attributeSet->SilvercartProductVariantAttributeSet()->type == 'attribute') {
                $attributes         = array();
                $selectedAttribute  = 0;

                foreach ($attributeSet->getAttributes() as $attribute) {
                    if ($attribute->isActive) {
                        if ($attribute->isDefault) {
                            $selectedAttribute = $attribute->ID;
                        }
                        $attributeName      = $attribute->name;
                        $fieldModifierNotes = $attribute->getFieldModifierNotes();

                        if (!empty($fieldModifierNotes)) {
                            $attributeName .= ' ('.$fieldModifierNotes.')';
                        }

                        $attributes[$attribute->ID] = $attributeName;
                    }
                }

                asort($attributes);

                $fieldType = 'DropdownField';
                
                if (!empty($attributeSet->useCustomFormField)) {
                    $fieldType = $attributeSet->useCustomFormField;
                }
                
                if ( $attributeSet->mustBeFilledIn &&
                    !empty($attributeSet->useCustomFormField)) {

                    $requirementMustBeFilledIn = array('callBack' => 'callback_'.$attributeSet->useCustomFormField);
                } else {
                    $requirementMustBeFilledIn = array();
                }
                
                $checkRequirements = array();
                    
                if ( $attributeSet->SilvercartProductVariantAttributeSet()->mustBeFilledIn &&
                    !empty($attributeSet->useCustomFormField)) {

                    $checkRequirements['callBack'] = 'callBack_'.$attributeSet->useCustomFormField;
                }
                $attributeSet->SilvercartProductVariantAttributeSet()->Attributes()->sort('name', 'ASC');
                    
                $callingObject->fieldGroups['SilvercartProductVariantAttributeSets']['SilvercartProductVariantAttributeSet'.$attributeSet->SilvercartProductVariantAttributeSet()->ID] = array(
                    'type'              => $fieldType,
                    'title'             => $attributeSet->SilvercartProductVariantAttributeSet()->name,
                    'value'             => $attributes,
                    'selectedValue'     => $selectedAttribute,
                    'attributeSet'      => $attributeSet,
                    'attributes'        => $attributeSet->Attributes(),
                    'silvercartProduct' => $silvercartProduct,
                    'checkRequirements' => $checkRequirements
                );
            }
            
            // ----------------------------------------------------------------
            // Attributes that allow user input
            // ----------------------------------------------------------------
            if ($attributeSet->SilvercartProductVariantAttributeSet()->type == 'userInput') {
                $dataSourceClassName = $attributeSet->SilvercartProductVariantAttributeSet()->dataSourceClass;
                
                if (class_exists($dataSourceClassName)) {
                    $dataSourceClass = new $dataSourceClassName();
                    
                    foreach ($dataSourceClass->getFormFields() as $formFieldName => $formFieldDefinition) {
                        $callingObject->fieldGroups['SilvercartProductVariantUserInputAttributeSets'][$formFieldName] = $formFieldDefinition;
                    }
                }
            }
        }
    }
    
    /**
     * Returns a string of HTML code containing fields to choose a variant.
     *
     * @param array &$arguments     The arguments to pass
     * @param mixed &$callingObject The calling object
     * 
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 16.11.2011
     */
    public function pluginAddCartFormDetailAdditionalFields(&$arguments, &$callingObject) {
        $silvercartProduct  = Controller::curr()->detailViewProduct;
        $renderer           = new ViewableData();
        $templateData       = array(
            'Form'                                      => $callingObject,
            'Controller'                                => Controller::curr(),
            'SilvercartAttributedVariantAttributeSets'  => $silvercartProduct->SilvercartAttributedVariantAttributeSets(),
            'SilvercartProduct'                         => $silvercartProduct
        );
        
        return $renderer->customise($templateData)->renderWith(
            'SilvercartProductVariantAddCartFormDetailPlugin'
        );
    }
}
