<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 */

/**
 * Adds information to newly created order positions.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 17.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantOrderPlugin extends DataObjectDecorator {

    /**
     * This method gets called while the SilvercartShoppingCartPositions are
     * converted to SilvercartOrderPositions.
     *
     * @param array &$shoppingCartPosition The SilvercartShoppingCartPosition
     * @param array &$orderPosition        The SilvercartOrderPosition
     * @param mixed &$callingObject        The calling object
     *
     * @return void
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 17.11.2011
     */
    public function pluginConvertShoppingCartPositionToOrderPosition(&$shoppingCartPosition, &$orderPosition, &$callingObject) {

        // --------------------------------------------------------------------
        // Add standard variant information to the position
        // --------------------------------------------------------------------
        $productVariantDefinition = '';

        $records = DB::query(
            sprintf(
                "
                SELECT
                    SilvercartProductVariantAttributeID,
                    SilvercartProductVariantAttributeSet
                FROM
                    SilvercartShoppingCartPosition_ProductVariantAttributes
                WHERE
                    SilvercartShoppingCartPositionID = %d
                ",
                $shoppingCartPosition->ID
            )
        );

        if ($shoppingCartPosition->ProductVariantAttributes()) {
            foreach ($shoppingCartPosition->ProductVariantAttributes() as $attribute) {

                $silvercartProductVariantAttributeSet = DataObject::get_by_id('SilvercartProductVariantAttributeSet', $attribute->SilvercartProductVariantAttributeSet);

                if ($silvercartProductVariantAttributeSet) {
                    $setName            = $silvercartProductVariantAttributeSet->name;
                    $fieldModifierNotes = $attribute->getFieldModifierNotes();

                    if ($fieldModifierNotes != '') {
                        $setName .= ' ('.$fieldModifierNotes.')';
                    }

                    $setName = '<i>'.$setName.'</i>';

                    if (!empty($productVariantDefinition)) {
                        $productVariantDefinition .= ', ';
                    }
                    $productVariantDefinition .= str_replace('<br />', ' ', $setName).': '.str_replace('<br />', ' ',$attribute->name);
                }
            }
        }

        // --------------------------------------------------------------------
        // Add user generated variant information to the position
        // --------------------------------------------------------------------
        if ($shoppingCartPosition->ProductVariantUGAttributes()) {
            foreach ($shoppingCartPosition->ProductVariantUGAttributes() as $ugAttribute) {
                $sourceClassName = $ugAttribute->inputObjectClassName;
                $sourceClassID   = $ugAttribute->inputObjectID;
                $sourceClassObj  = DataObject::get_by_id($sourceClassName, $sourceClassID);

                if ($sourceClassObj) {
                    $shoppingCartPositionInfo = $sourceClassObj->getShoppingCartInfo();

                    if ($shoppingCartPositionInfo) {
                        if (!empty($productVariantDefinition)) {
                            $productVariantDefinition .= ', ';
                        }
                        $productVariantDefinition .= '<em>'.str_replace('<br />', ' ',$shoppingCartPositionInfo->setName).'</em>: '.str_replace('<br />', ' ',$shoppingCartPositionInfo->name);
                    }
                }
            }

            // Delete user generated attributes after conversion to order position
            foreach ($shoppingCartPosition->ProductVariantUGAttributes() as $ugAttribute) {
                $ugAttribute->delete();
            }
        }

        $orderPosition->productVariantDefinition = $productVariantDefinition;
    }
}
