<?php
/**
 * Copyright 2011 pixeltricks GmbH
 *
 * This file is part of SilverCart.
 *
 * SilverCart is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SilverCart is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SilverCart.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 */

/**
 * Delivers additional information for SilvercartShoppingCart.
 *
 * @package SilvercartProductVariants
 * @subpackage Plugins
 * @author Sascha Koehler <skoehler@pixeltricks.de>
 * @since 16.11.2011
 * @license http://www.gnu.org/licenses/lgpl.html GNU Lesser General Public License
 * @copyright 2011 pixeltricks GmbH
 */
class SilvercartProductVariantShoppingCartPlugin extends DataObjectDecorator {
    
    /**
     * The field handlers for processing special fields
     *
     * @var array
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 07.12.2011
     */
    public static $registeredFieldHandlers = array();
    
    /**
     * Registers a field handler.
     *
     * @param mixed $handlerObject The handler object
     *
     * @return void
     * 
     * @author Sascha Koehler <skoehler@pixeltricks.de>
     * @since 07.12.2011
     */
    public static function registerFieldHandler($handlerObject) {
        self::$registeredFieldHandlers[] = $handlerObject;
    }
    
    /**
     * Overwrites the addProduct method.
     *
     * @param array &$arguments     The arguments to pass
     *                              $arguments[0]: formData
     * @param mixed &$callingObject The calling object
     * 
     * @return string
     *
     * @author Sascha Koehler <skoehler@pixeltricks.de>, Sebastian Diel <sdiel@pixeltricks.de>
     * @since 30.11.2012
     */
    public function pluginOverwriteAddProduct(&$arguments, &$callingObject) {
        $overwritten = false;
        if ($arguments[0]['productID'] && $arguments[0]['productQuantity']) {
            $member     = Member::currentUser();
            if (!$member) {
                $member = SilvercartCustomer::createAnonymousCustomer();
            }
            $cart = $member->getCart();
            if ($cart) {
                $product = DataObject::get_by_id('SilvercartProduct', $arguments[0]['productID'], 'Created');
                if ($product->hasProductVariants()) {
                    $overwritten = true;
                    $attributes = array();

                    foreach ($arguments[0] as $fieldName => $fieldValue) {
                        if (substr($fieldName, 0, 36) == 'SilvercartProductVariantAttributeSet') {
                            if (!empty($fieldValue)) {
                                $attributeSetID = substr($fieldName, 36);
                                $attributes[$attributeSetID] = $fieldValue;
                            }
                        }

                        foreach (self::$registeredFieldHandlers as $fieldHandler) {
                            if (method_exists($fieldHandler, 'processRequest')) {
                                $processedAttributes = $fieldHandler->processRequest();

                                if (!empty($processedAttributes)) {
                                    foreach ($processedAttributes as $attributeSetID => $attributeID) {
                                        $attributes[$attributeSetID] = $attributeID;
                                    }
                                }
                            }
                        }
                    }

                    if ($product) {
                        $arguments[0]['productQuantity'] = str_replace(',', '.', $arguments[0]['productQuantity']);
                        $quantity                        = (float) $arguments[0]['productQuantity'];

                        if ($quantity > 0) {
                            $product->SVTLFaddToCartWithAttributes($cart->ID, $quantity, $attributes);
                        }
                    }
                }
            }
        }
        
        return $overwritten;
    }
}
